Package: purrrlyr
Title: Tools at the Intersection of 'purrr' and 'dplyr'
Version: 0.0.10
Authors@R: 
    c(person(given = "Lionel",
             family = "Henry",
             role = c("aut", "cre"),
             email = "lionel@posit.co"),
      person(given = "Hadley",
             family = "Wickham",
             role = "ctb",
             email = "hadley@posit.co"),
      person("Posit Software, PBC", role = c("cph", "fnd"),
             comment = c(ROR = "03wc8by49")))
Description: Some functions at the intersection of 'dplyr' and
    'purrr' that formerly lived in 'purrr'.
License: GPL-3 | file LICENSE
URL: https://github.com/hadley/purrrlyr
BugReports: https://github.com/hadley/purrrlyr/issues
Imports: dplyr (>= 0.8.0), magrittr (>= 1.5), purrr (>= 0.2.2), Rcpp
Suggests: covr, testthat (>= 3.0.0)
LinkingTo: Rcpp
Encoding: UTF-8
RoxygenNote: 7.3.3
Config/testthat/edition: 3
Config/build/compilation-database: true
NeedsCompilation: yes
Packaged: 2025-09-11 15:04:28 UTC; lionel
Author: Lionel Henry [aut, cre],
  Hadley Wickham [ctb],
  Posit Software, PBC [cph, fnd] (ROR: <https://ror.org/03wc8by49>)
Maintainer: Lionel Henry <lionel@posit.co>
Repository: CRAN
Date/Publication: 2025-09-12 05:30:11 UTC
Built: R 4.6.0; aarch64-apple-darwin23; 2026-04-26 12:07:10 UTC; unix
Archs: purrrlyr.so.dSYM
