# Generated by roxygen2: do not edit by hand

export("%>%")
export(by_row)
export(by_slice)
export(dmap)
export(dmap_at)
export(dmap_if)
export(invoke_rows)
export(map_rows)
export(slice_rows)
export(unslice)
importFrom(Rcpp,sourceCpp)
importFrom(dplyr,group_data)
importFrom(magrittr,"%>%")
useDynLib(purrrlyr, .registration = TRUE)
