# purrrlyr

<!-- badges: start -->
[![Lifecycle: superseded](https://img.shields.io/badge/lifecycle-superseded-blue.svg)](https://lifecycle.r-lib.org/articles/stages.html)
[![CRAN_Status_Badge](http://www.r-pkg.org/badges/version/purrrlyr)](https://cran.r-project.org/package=purrrlyr)
[![R-CMD-check](https://github.com/hadley/purrrlyr/actions/workflows/R-CMD-check.yaml/badge.svg)](https://github.com/hadley/purrrlyr/actions/workflows/R-CMD-check.yaml)
<!-- badges: end -->

purrrlyr contains some functions that lie at the intersection of purrr
and dplyr. They have been removed from purrr in order to make the
package lighter and because they have been replaced by other solutions
in the tidyverse.

Please see Jenny Brian's
[webinar on row-oriented workflows](https://github.com/jennybc/row-oriented-workflows#readme)
for some alternative approaches.
